/***************************************************************
 search.m

 MAGMA code to seive for all f of level <= 1000 that satify
 both conditions 1 and 2, given the table 
       http://modular.fas.harvard.edu/Tables/an_1-1000.gp

 William Stein and David Savitt
 19 February 2001.

 ***************************************************************/

function SatisfiesConditionOne(N, f)
   return Degree(f[1]) eq 2;
end function;

function SatisfiesConditionTwoAtEll(N, f, ell)
   if not IsPrime(ell) or ell gt 5 then 
      return false;
   end if;
   if N mod ell^2 ne 0 then
      return false;
   end if;
   if N mod ell^3 eq 0 then
      return false;
   end if;

   K := NumberField(f[1]);
   O := MaximalOrder(K);
   I := ideal<O | ell>;
   F := Factorization(I);
   if #F eq 1 and F[1][2] eq 1 then
      return false;
   end if;

   return true;
end function;

function SatisfiesConditionTwo(N, f)
   return [ell : ell in [3,5] | SatisfiesConditionTwoAtEll(N, f,ell)];
end function;

procedure ListExamples(Nrange, table)
   printf "examples := [\n";
   for N in Nrange do
      for A in table[N] do
         f := <A[3], A[4]>;
         if SatisfiesConditionOne(N, f) then
            two := SatisfiesConditionTwo(N,f);
            if #two gt 0 then
               printf "<%o, %o, %o, %o>,\n", N, two, A[1], Discriminant(A[3]);
            end if;
         end if;
      end for;
   end for;
   printf "];\n";
end procedure;

function ModularKernels(examples)
   i := 1;
   ans := "";
   while i le #examples do
      A := examples[i][2];
      N := A[1];
      M := ModularSymbols(N);
      D := SortDecomposition(NewformDecomposition(
                      NewSubspace(CuspidalSubspace(M)))
           );
      while A[1] eq N do
         ans := ans cat Sprintf("ModularKernel[%o] := %o; // N = %o\n", examples[i][1], 
				Invariants(ModularKernel(D[A[3]])), N);
         i +:= 1;
         if i gt #examples then
            break;
         end if;
         A := examples[i][2];
      end while;
      DisownChildren(M);
   end while;
   return ans;
end function;


